from . import config
from metakernel import Magic
from IPython.display import HTML
from licofage.api import Study
from ratser.walimp import from_walnut
from ratser.trick import trick, to_walnut
from pathlib import Path as P


def genparentdir(fname):
    parent = fname.parent
    if not parent.is_dir():
        parent.mkdir(parents=True)
    return fname


class MyMagic(Magic):
    def line_showme(self, lapin):
        """
        %showme LAPIN - display Result named lapin.gv
        """
        try:
            import pydot
        except:
            raise Exception("You need to install pydot")
        graph = pydot.graph_from_dot_file(
            config.WALNUT_HOME / "Result" / f"{str(lapin)}.gv"
        )[0]
        svg = graph.create_svg()
        if hasattr(svg, "decode"):
            svg = svg.decode("utf-8")
        html = HTML(svg)
        self.kernel.Display(html)

    def line_DT(self, name, subst, bound=None):
        """
        %DT blop "a->ab, b->a"
        %DT blop "a->aba, b->b" 30
        %DT [name] [subst] [bound]

        Compute the Dumont-Thomas numeration system associated to
        the substitution "a->ab, b->a" with its addition. If not
        X-Pisot, a bound is needed.

        Generate a DFAO Blop for the fixpoint of the substitution
        and a numeration system ?msd_blop

        A script check_blop.txt is provided to check the addition.
        """
        try:
            if bound is not None:
                bound = [int(bound)]
            outdir = config.WALNUT_HOME
            MSD = True
            endian = "msd_" if MSD else "lsd_"
            format = "Walnut"
            stud = Study(subst, True, True, True)
            dfao = P("Word Automata Library") / P(f"{name.title()}.txt")
            parent = P("Word Automata Library") / P(f"{name.title()}Parent.txt")
            numsys = P("Custom Bases") / P(f"{endian}{name}.txt")
            addition = P("Custom Bases") / P(f"{endian}{name}_addition.txt")
            check = P("Command Files") / P(f"check_{name}.txt")
            stud.gendfao(genparentdir(outdir / dfao), format)
            stud.genparentdfao(genparentdir(outdir / parent), format)
            stud.gennumsys(
                genparentdir(outdir / numsys),
                MSD,
                format,
                True,
            )
            (aa, b, fname) = ([1, 1, -1], 0, addition)
            stud.genlinear(
                genparentdir(outdir / fname),
                (aa, b),
                MSD,
                format,
                True,
                None,
                None,
                None,
                bound,
                False,
            )
            stud.gencheck(genparentdir(outdir / check), MSD, name)
            print("Done.")
        except ValueError as e:
            print(f"*** {str(e)}. Sorry!")
        except AssertionError as e:
            print(f"### {str(e)}. Sorry!")

    def line_SGT(self, maple, ns, dfao):
        """
        %SGT lr_fib msd_fib Comp_fib
        %SGT [maple] [ns] [dfao]

        Compute the DFAO [dfao] using the semigroup trick from
        the linear representation of a rational serie given in
        Maple file [maple] (generated by Walnut).

        The output DFAO declares using the numeration system [ns].

        Input file should be Result/[maple].mpl

        Output file will be Word Automata Library/[dfao].txt
        """
        outdir = config.WALNUT_HOME
        infile = P("Result") / P(f"{maple}.mpl")
        outfile = P("Word Automata Library") / P(f"{dfao}.txt")
        print(f"Reading {infile}")
        with open(outdir / infile) as f:
            s = from_walnut(f)
        t = s.minimize()
        print(f"Rank: {t.rank()}")
        st = trick(t)
        print(f"{len(st)} states")
        print("values:", sorted(list(set(x[1] for _, x in st.items()))))
        print(f"Writing {outfile}")
        with open(outdir / outfile, "w") as f:
            to_walnut(st, ns, f)
        print("Done.")

    def line_EQ(self, maple1, maple2):
        """
        %EQ lr_f lr_g
        %EQ [maple1] [maple2]

        Test if the rational series given in the Maple files
        [maple1] and [maple2] (generated by Walnut) are equal.

        Input files should be Result/[maple?].mpl
        """
        outdir = config.WALNUT_HOME
        infile = P("Result") / P(f"{maple1}.mpl")
        print(f"Reading {infile}")
        with open(outdir / infile) as f:
            s = from_walnut(f)
        t = s.minimize()
        print(f"Rank: {t.rank()}")
        infile = P("Result") / P(f"{maple2}.mpl")
        print(f"Reading {infile}")
        with open(outdir / infile) as f:
            ss = from_walnut(f)
        tt = ss.minimize()
        print(f"Rank: {tt.rank()}")
        print(f"Equality: { t == tt }")
